unit RoboUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms, OleAuto, RR_COM_APILib_TLB,
  Dialogs, StdCtrls, ExtCtrls, Buttons, StrUtils, AppEvnts;

type
  TVariantArray = array of Variant;
  TForm36 = class(TForm)
    Panel1: TPanel;
    Splitter1: TSplitter;
    Panel2: TPanel;
    LabeledEdit1: TLabeledEdit;
    LabeledEdit2: TLabeledEdit;
    AnalyzeB: TButton;
    LogsM: TMemo;
    LabeledEdit3: TLabeledEdit;
    GetImageSB: TSpeedButton;
    OpenDialogPicture: TOpenDialog;
    LabeledEdit5: TLabeledEdit;
    GetRoboFileSB: TSpeedButton;
    OpenDialogRoboBarcode: TOpenDialog;
    RoboRealmExecution: TTimer;
    ApplicationEvents: TApplicationEvents;
    Button1: TButton;
    LabeledEdit4: TLabeledEdit;
    Analyze2B: TButton;
    procedure FormDestroy(Sender: TObject);
    procedure ApplicationEventsException(Sender: TObject; E: Exception);
    procedure GetImageSBClick(Sender: TObject);
    procedure GetRoboFileSBClick(Sender: TObject);
    procedure RoboRealmExecutionTimer(Sender: TObject);
    procedure AnalyzeBClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Analyze2BClick(Sender: TObject);
  private
    procedure LogMessage(MssgToLog: string);
    procedure SetupRoboRealm;
    function ExistRoboRealmOLE: Boolean;
    function CreateRoboRealmOLE: Boolean;
    function ConnectToRoboRealmServer: Boolean;
    function ConnectedToRoboRealmServer: Boolean;
    procedure DisconnectFromRoboRealmServer;
    procedure DestroyRoboRealmOLE;
    function GetRoboRealmVersion: string;
    function GetRoboRealmProgram: string;
    function SetRoboRealmNewPipeline: Boolean;
    function GetRoboRealmModuleTime(ModuleName: string; ModuleNumber: Integer; var ModuleTime: Integer): Boolean;
    function ProgramLoaded: Boolean;
    procedure SetExecutionTimer(Value,Interval: Integer);
    procedure LinerExecution;
    function RoboStringParsing(ResultValues :string; WidthDim, HeightDim : Integer) : string;
  public
    { Public declarations }
  end;

var
  Form36: TForm36;

  RoboRealmOLE:           Variant;
  RoboRealmServerAddress: Variant;
  RoboRealmServerPort:    Variant;
  RoboRealmImageFile:     Variant;
  RoboRealmProgramFile:   Variant;
  WaitImageForDuration:   Integer;
  ExecutionDuration:      LongWord;
  AnalysisDuration:       LongWord;
  RR: TAPI_Wrapper;

implementation

{$R *.dfm}

procedure TForm36.FormCreate(Sender: TObject);
begin
  RR := TAPI_Wrapper.Create(Self);
end;

procedure TForm36.FormDestroy(Sender: TObject);
begin
  if RR.IsConnected then RR.Disconnect;
  RR.Free;
  DestroyRoboRealmOLE;
end;

procedure TForm36.LogMessage(MssgToLog: string);
begin
  LogsM.Lines.Add(FormatDateTime('dd.mm.yyyy hh:nn:ss.zzz',now) + ' ' + MssgToLog);
end;

procedure TForm36.ApplicationEventsException(Sender: TObject; E: Exception);
begin
  LogMessage('Unhandled exception '+E.Message);
end;

procedure TForm36.SetupRoboRealm;
begin
  if LabeledEdit1.Text = '' then RoboRealmServerAddress := 'localhost'
                            else RoboRealmServerAddress := LabeledEdit1.Text;
  if LabeledEdit2.Text = '' then RoboRealmServerPort    := 6060
                            else RoboRealmServerPort    := StrToInt(LabeledEdit2.Text);
  if LabeledEdit3.Text = '' then RoboRealmImageFile     := 'C:\Program Files (x86)\RoboRealm\uB_OMR_Images\Temporary.jpg'
                            else RoboRealmImageFile     := LabeledEdit3.Text;
  if LabeledEdit5.Text = '' then RoboRealmProgramFile   := 'C:\Program Files (x86)\RoboRealm\uB_OMR_Scripts\uBWorksheet.robo'
                            else RoboRealmProgramFile   := LabeledEdit5.Text;
  WaitImageForDuration := StrToIntDef(LabeledEdit4.Text,2500);
end;

function TForm36.ExistRoboRealmOLE: Boolean;
begin
  Result := VarType(RoboRealmOLE) <> varEmpty;
end;

function TForm36.CreateRoboRealmOLE: Boolean;
begin
  Result := False;
  if not ExistRoboRealmOLE then begin
    RoboRealmOLE := CreateOleObject('RoboRealm.API.1');
    if not ExistRoboRealmOLE then begin
      LogMessage('Unable to create robo-realm OLE object. Check if Robo-Realm DLL (RR_COM_API.dll) is registered');
    end else begin
      Result := True;
      LogMessage('Successfuly created robo-realm OLE object');
    end;
  end else begin
    Result := True;
    LogMessage('Robo-realm OLE object already created');
  end;
end;

function TForm36.ConnectToRoboRealmServer: Boolean;
begin
  Result := False;
  CreateRoboRealmOLE;
  if ExistRoboRealmOLE then begin
    if not RoboRealmOLE.IsConnected then begin
      SetupRoboRealm;
      if not RoboRealmOLE.Connect(VarToStr(RoboRealmServerAddress),RoboRealmServerPort) then begin
        LogMessage('Unable to connect to robo-realm server at '+VarToStr(RoboRealmServerAddress)+':'+VarToStr(RoboRealmServerPort));
      end else begin
        LogMessage('Connected to robo-realm server');
        Result := True;
      end;
    end else begin
      LogMessage('Already connected to robo-realm server at '+VarToStr(RoboRealmServerAddress)+':'+VarToStr(RoboRealmServerPort));
      Result := True;
    end;
  end else begin
    LogMessage('Robo-realm OLE object not created');
  end;
end;

function TForm36.ConnectedToRoboRealmServer: Boolean;
begin
  Result := False;
  if ExistRoboRealmOLE then Result := RoboRealmOLE.IsConnected;
end;

procedure TForm36.DisconnectFromRoboRealmServer;
begin
  if ConnectedToRoboRealmServer then RoboRealmOLE.Disconnect;
end;

procedure TForm36.DestroyRoboRealmOLE;
begin
  DisconnectFromRoboRealmServer;
  if ExistRoboRealmOLE then RoboRealmOLE := Unassigned;
end;

function TForm36.GetRoboRealmVersion: string;
begin
  Result := '';
  if ConnectedToRoboRealmServer then Result := RoboRealmOLE.GetVersion
                                else LogMessage('Unable to get version due to Robo-realm OLE object not created or disconnected');
end;

function TForm36.GetRoboRealmProgram: string;
begin
  Result := '';
  if ConnectedToRoboRealmServer then Result := RoboRealmOLE.GetProgram
                                else LogMessage('Unable to get active program due to Robo-realm OLE object not created or disconnected');
end;

function TForm36.SetRoboRealmNewPipeline: Boolean;
begin
  Result := True;
  if ConnectedToRoboRealmServer then RoboRealmOLE.NewPipeline
                                else begin
                                       Result := False;
                                       LogMessage('Unable to set new pipeline due to Robo-realm OLE object not created or disconnected');
                                     end;
end;

function TForm36.GetRoboRealmModuleTime(ModuleName: string; ModuleNumber: Integer; var ModuleTime: Integer): Boolean;
begin
  Result := False;
  if ConnectedToRoboRealmServer then begin
    ModuleTime := RoboRealmOLE.GetModuleTime(ModuleName,ModuleNumber);
    Result := True;
  end else LogMessage('Unable to set new pipeline due to Robo-realm OLE object not created or disconnected');
end;

function TForm36.ProgramLoaded: Boolean;
var
  CurrentProgram: string;
  ProgramToRun:   string;
begin
  CurrentProgram := GetRoboRealmProgram;
  ProgramToRun   := ChangeFileExt(ExtractFileName(RoboRealmProgramFile),'');
  LogMessage('Current RR program is '+Copy(CurrentProgram,1,50));
  LogMessage('Expected RR program is '+ProgramToRun);
  Result := ContainsText(CurrentProgram,ProgramToRun);
end;

procedure TForm36.GetImageSBClick(Sender: TObject);
begin
  if OpenDialogPicture.Execute then LabeledEdit3.Text := OpenDialogPicture.FileName;
end;

procedure TForm36.GetRoboFileSBClick(Sender: TObject);
begin
  if OpenDialogRoboBarcode.Execute then LabeledEdit5.Text := OpenDialogRoboBarcode.FileName;
end;

procedure TForm36.SetExecutionTimer(Value,Interval: Integer);
begin
  RoboRealmExecution.Tag := Value;
  RoboRealmExecution.Interval := Interval;
  RoboRealmExecution.Enabled := True;
end;

procedure TForm36.RoboRealmExecutionTimer(Sender: TObject);
var
  Reply:            Integer;
  VariableResults:  OleVariant;
  VariableResult:   Variant;
  i,j:              Integer;
  SampleID:         string;
  PageID:           string;
  VersionNumber:    string;
  ProcessingErrors: string;
  ProcessStatus:    string;
  ImageCount:       string;
  ImageProcessed:   string;
  ProcessTime:      string;
  RunTime:          string;
  Results:          string;
  PhaseStart:      Integer;
  PhaseStop:       Integer;
  LoadedProgramName: string;
  LoadedImageName:   string;
begin
  RoboRealmExecution.Enabled := False;
  case RoboRealmExecution.Tag of
    0:   begin
           AnalyzeB.Enabled := False;
           AnalysisDuration := GetTickCount;
           WaitImageForDuration := StrToIntDef(LabeledEdit4.Text,2500);
           LogMessage('Start');
           Reply := RoboRealmOLE.Run('off');
           LogMessage('Program stopped to make sure we have correct start condition. Stop reply '+IntToStr(Reply));

           LogMessage('Load requested');
           PhaseStart := GetTickCount;
           Reply := RoboRealmOLE.LoadProgram(VarToStr(RoboRealmProgramFile));
           PhaseStop := GetTickCount;
           LogMessage('Loading of program reply '+IntToStr(Reply)+' loading phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
           SetExecutionTimer(1,5000);
         end;
    1:   begin
           PhaseStart := GetTickCount;
           LoadedProgramName := RoboRealmOLE.GetVariable('ROBO_PATH');
           PhaseStop := GetTickCount;
           LogMessage('Loaded program reply '+LoadedProgramName+' loading phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
           SetExecutionTimer(2,5000);
         end;
    2:   begin
           LogMessage('Image loading');

           PhaseStart := GetTickCount;
           Reply := RoboRealmOLE.LoadImage('source',VarToStr(RoboRealmImageFile));
           PhaseStop := GetTickCount;
           LogMessage('Loading of image requested. Loading reply '+IntToStr(Reply)+' loading phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));

           SetExecutionTimer(3,5000);
         end;
    3:   begin
           PhaseStart := GetTickCount;
           LoadedImageName := RoboRealmOLE.GetVariable('IMAGE_PATH');
           PhaseStop := GetTickCount;
           LogMessage('Loaded program reply '+LoadedImageName+' loading phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));

           LogMessage('Program execution start');
           SetExecutionTimer(4,100);
           ExecutionDuration := GetTickCount;
           try
             PhaseStart := GetTickCount;
             Reply := RoboRealmOLE.Run('on');
             PhaseStop := GetTickCount;
             LogMessage('Program run requested. Run reply '+IntToStr(Reply)+' run "ON" phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
           except
             on E: Exception do LogMessage('Exception when program run '+E.Message);
           end;
         end;
    4:   begin
           try
             LogMessage('Program about to wait for image to be processed');
             PhaseStart := GetTickCount;
             Reply := RoboRealmOLE.WaitImageFor(WaitImageForDuration);
             PhaseStop := GetTickCount;
             LogMessage('Image wait ended. Wait reply '+IntToStr(Reply)+' "wait image" phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));

             PhaseStart := GetTickCount;
             Reply := RoboRealmOLE.Run('off');
             PhaseStop := GetTickCount;
             LogMessage('Program stopped. Stop reply '+IntToStr(Reply)+' off duration '+' run "OFF" phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
           except
             on E: Exception do LogMessage('Exception while stopping program run '+E.Message);
           end;
           ExecutionDuration := GetTickCount - ExecutionDuration;
           SetExecutionTimer(5,100);
         end;
    5:   begin
           LogMessage('Getting variables');
           //SampleID         := RoboRealmOLE.GetVariable('SampleID');
           //PageID           := RoboRealmOLE.GetVariable('PageID');
           //VersionNumber    := RoboRealmOLE.GetVariable('VersionNumber');
           //ProcessingErrors := RoboRealmOLE.GetVariable('ProcessingErrors');
           //ProcessStatus    := RoboRealmOLE.GetVariable('ProcessStatus');
           //ImageProcessed   := RoboRealmOLE.GetVariable('IMAGE_PROCESSED');
           //ImageCount       := RoboRealmOLE.GetVariable('IMAGE_COUNT');
           //ProcessTime      := RoboRealmOLE.GetVariable('PROCESS_TIME');
           //RunTime          := RoboRealmOLE.GetVariable('RUN_TIME');
           PhaseStart := GetTickCount;
           VariableResults  := RoboRealmOLE.GetVariables('SampleID,PageID,VersionNumber,ProcessingErrors,ProcessStatus,IMAGE_PROCESSED,IMAGE_COUNT,PROCESS_TIME,RUN_TIME,Table11Status,Table12Status,Table21Status,Table22Status,Table23Status,Table24Status');
           PhaseStop := GetTickCount;
           LogMessage('Variables got '+Format('%d ms',[PhaseStop - PhaseStart]));

           //LogMessage('Sample ID         '+SampleID);
           //LogMessage('Page   ID         '+PageID);
           //LogMessage('Version number    '+VersionNumber);
           //LogMessage('Processing errors '+ProcessingErrors);
           //LogMessage('Process status    '+ProcessStatus);
           //LogMessage('Image processed   '+ImageProcessed);
           //LogMessage('Image count       '+ImageCount);
           //LogMessage('Process time      '+ProcessTime);
           //LogMessage('run time          '+RunTime);

           if not VarIsEmpty(VariableResults) then begin
             i := VarArrayLowBound(VariableResults,1);
             j := VarArrayHighBound(VariableResults,1);
             while i <= j do begin
               VariableResult := VariableResults[i];
               LogMessage('Variable '+IntToStr(i)+' has value '+VariableResult);
               if i = 1 then PageID := VariableResult;
               Inc(i);
             end;
           end;

           if PageID = '1' then begin
             PhaseStart := GetTickCount;
             Results := RoboRealmOLE.GetVariable('niz_out11');
             PhaseStop := GetTickCount;
             LogMessage(#13'Page 1 - table 1 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
             LogMessage('Page 1 - table 1 '+RoboStringParsing(Results,6,24));


             PhaseStart := GetTickCount;
             Results := RoboRealmOLE.GetVariable('niz_out12');
             PhaseStop := GetTickCount;
             LogMessage(#13'Page 1 - table 2 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
             LogMessage('Page 1 - table 2 '+RoboStringParsing(Results,6,24));

           end else if PageID = '2' then begin
             PhaseStart := GetTickCount;
             Results := RoboRealmOLE.GetVariable('niz_out21');
             PhaseStop := GetTickCount;
             LogMessage(#13'Page 2 - table 1 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
             LogMessage('Page 2 - table 1 '+RoboStringParsing(Results,6,22));

             PhaseStart := GetTickCount;
             Results := RoboRealmOLE.GetVariable('niz_out22');
             PhaseStop := GetTickCount;
             LogMessage(#13'Page 2 - table 2 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
             LogMessage('Page 2 - table 2 '+RoboStringParsing(Results,6,22));

             PhaseStart := GetTickCount;
             Results := RoboRealmOLE.GetVariable('niz_out23');
             PhaseStop := GetTickCount;
             LogMessage(#13'Page 2 - table 3 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
             LogMessage('Page 2 - table 3 '+RoboStringParsing(Results,6,24));

             PhaseStart := GetTickCount;
             Results := RoboRealmOLE.GetVariable('niz_out24');
             PhaseStop := GetTickCount;
             LogMessage(#13'Page 2 - table 4 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
             LogMessage('Page 2 - table 4 '+RoboStringParsing(Results,6,24));
           end;

           SetExecutionTimer(100,10);
         end;

    100: begin
           AnalysisDuration := GetTickCount - AnalysisDuration;
           LogMessage('Stop');
           LogMessage(#13'Execution duration '+Format('%5.2f',[ExecutionDuration / 1000]));
           LogMessage('Analysis duration '+Format('%5.2f',[AnalysisDuration / 1000]));
           AnalyzeB.Enabled := True;
         end;
  end;
end;

procedure TForm36.AnalyzeBClick(Sender: TObject);
begin
  if ConnectToRoboRealmServer then SetExecutionTimer(0,10);
end;

procedure TForm36.Button1Click(Sender: TObject);
begin
  LogsM.Lines.Clear;
end;

procedure TForm36.Analyze2BClick(Sender: TObject);
begin
  if ConnectToRoboRealmServer then LinerExecution;
end;

procedure TForm36.LinerExecution;
var
  Reply:            Integer;
  VariableResults:  OleVariant;
  VariableResult:   Variant;
  i,j:              Integer;
  SampleID:         string;
  PageID:           string;
  VersionNumber:    string;
  ProcessingErrors: string;
  ProcessStatus:    string;
  ImageCount:       string;
  ImageProcessed:   string;
  ProcessTime:      string;
  RunTime:          string;
  Results:          string;
  PhaseStart:      Integer;
  PhaseStop:       Integer;
begin
           AnalyzeB.Enabled := False;
           AnalysisDuration := GetTickCount;
           WaitImageForDuration := StrToIntDef(LabeledEdit4.Text,2500);
           LogMessage('Start');
           Reply := RoboRealmOLE.Run('off');
           LogMessage('Program stopped to make sure we have correct start condition. Stop reply '+IntToStr(Reply));
           if Boolean(Reply) then begin

             LogMessage('Program execution start');
             ExecutionDuration := GetTickCount;
             try
               PhaseStart := GetTickCount;
               Reply := RoboRealmOLE.Run('on');
               PhaseStop := GetTickCount;
               LogMessage('Program run requested. Run reply '+IntToStr(Reply)+' run "ON" phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
             except
               on E: Exception do LogMessage('Exception when program run '+E.Message);
             end;

             if Boolean(Reply) then begin

               try
                 LogMessage('Program about to wait for image to be processed');
                 PhaseStart := GetTickCount;
                 Reply := RoboRealmOLE.WaitImageFor(WaitImageForDuration);
                 PhaseStop := GetTickCount;
                 LogMessage('Image wait ended. Wait reply '+IntToStr(Reply)+' "wait image" phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));

                 PhaseStart := GetTickCount;
                 Reply := RoboRealmOLE.Run('off');
                 PhaseStop := GetTickCount;
                 LogMessage('Program stopped. Stop reply '+IntToStr(Reply)+' off duration '+' run "OFF" phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
               except
                 on E: Exception do LogMessage('Exception while stopping program run '+E.Message);
               end;
               ExecutionDuration := GetTickCount - ExecutionDuration;

               if Boolean(Reply) then begin

                 LogMessage('Getting variables');

                 PhaseStart := GetTickCount;
                 VariableResults  := RoboRealmOLE.GetVariables('SampleID,PageID,VersionNumber,ProcessingErrors,ProcessStatus,IMAGE_COUNT,RUN_TIME,Table11Status,Table12Status,Table21Status,Table22Status,Table23Status,Table24Status');
                 PhaseStop := GetTickCount;
                 LogMessage('Variables got '+Format('%d ms',[PhaseStop - PhaseStart]));

                 if not VarIsEmpty(VariableResults) then begin
                   i := VarArrayLowBound(VariableResults,1);
                   j := VarArrayHighBound(VariableResults,1);
                   while i <= j do begin
                     VariableResult := VariableResults[i];
                     LogMessage('Variable '+IntToStr(i)+' has value '+VariableResult);
                     if i = 1 then PageID := VariableResult;
                     Inc(i);
                   end;
                 end;

                 if PageID = '1' then begin
                   PhaseStart := GetTickCount;
                   Results := RoboRealmOLE.GetVariable('niz_out11');
                   PhaseStop := GetTickCount;
                   LogMessage(#13'Page 1 - table 1 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
                   LogMessage('Page 1 - table 1 '+RoboStringParsing(Results,6,24));


                   PhaseStart := GetTickCount;
                   Results := RoboRealmOLE.GetVariable('niz_out12');
                   PhaseStop := GetTickCount;
                   LogMessage(#13'Page 1 - table 2 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
                   LogMessage('Page 1 - table 2 '+RoboStringParsing(Results,6,24));

                 end else if PageID = '2' then begin
                   PhaseStart := GetTickCount;
                   Results := RoboRealmOLE.GetVariable('niz_out21');
                   PhaseStop := GetTickCount;
                   LogMessage(#13'Page 2 - table 1 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
                   LogMessage('Page 2 - table 1 '+RoboStringParsing(Results,6,22));

                   PhaseStart := GetTickCount;
                   Results := RoboRealmOLE.GetVariable('niz_out22');
                   PhaseStop := GetTickCount;
                   LogMessage(#13'Page 2 - table 2 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
                   LogMessage('Page 2 - table 2 '+RoboStringParsing(Results,6,22));

                   PhaseStart := GetTickCount;
                   Results := RoboRealmOLE.GetVariable('niz_out23');
                   PhaseStop := GetTickCount;
                   LogMessage(#13'Page 2 - table 3 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
                   LogMessage('Page 2 - table 3 '+RoboStringParsing(Results,6,24));

                   PhaseStart := GetTickCount;
                   Results := RoboRealmOLE.GetVariable('niz_out24');
                   PhaseStop := GetTickCount;
                   LogMessage(#13'Page 2 - table 4 phase duration '+Format('%d ms',[PhaseStop - PhaseStart]));
                   LogMessage('Page 2 - table 4 '+RoboStringParsing(Results,6,24));
                 end;

                 AnalysisDuration := GetTickCount - AnalysisDuration;
                 LogMessage('Stop');
                 LogMessage(#13'Execution duration '+Format('%5.2f',[ExecutionDuration / 1000]));
                 LogMessage('Analysis duration '+Format('%5.2f',[AnalysisDuration / 1000]));

               end else begin
                 LogMessage('Program stop failed - it is not possible to get correct variable values unless we stop program run');
               end;

             end;

           end;
           AnalyzeB.Enabled := True;
end;

function TForm36.RoboStringParsing(ResultValues :string; WidthDim, HeightDim : Integer) : string;
var
  ResultValuesParsed: string;
  I,J, CellMarksCounter: Integer;
begin
        // Parsiranje result niza iz roborealma
        ResultValuesParsed := '';
        I := 1;

        while (I < Length(ResultValues)) do begin //18
          for J := 1 to (WidthDim Div 2)-2 do begin //16
            CellMarksCounter := 0;
            if ResultValues[I]='0' then begin
              ResultValuesParsed := ResultValuesParsed + '0,-,';
              CellMarksCounter := CellMarksCounter + 2;
            end;
            if ResultValues[I+2]='2' then begin
              if CellMarksCounter = 0 then
              ResultValuesParsed := ResultValuesParsed + '2,-,';
              CellMarksCounter := CellMarksCounter + 2;
            end;
            if ResultValues[I+2+(2*WidthDim)]='3' then begin
              if CellMarksCounter = 0 then
              ResultValuesParsed := ResultValuesParsed + '3,-,';
              CellMarksCounter := CellMarksCounter + 2;
            end;
            if ResultValues[I+(2*WidthDim)]='p' then begin
              if CellMarksCounter = 0 then begin
                ResultValuesParsed := ResultValuesParsed + 'X,-,';
              end else begin
                if CellMarksCounter = 2 then begin
                  Delete(ResultValuesParsed,Length(ResultValuesParsed)-1,2);
                  ResultValuesParsed := ResultValuesParsed + 'p,';
                end;
              end;
              Inc(CellMarksCounter);
            end;
            if (CellMarksCounter=0) then begin
              ResultValuesParsed := ResultValuesParsed + '-,-,';
            end;
            if (CellMarksCounter > 3) then begin
              Delete(ResultValuesParsed,Length(ResultValuesParsed)-3,4);
              ResultValuesParsed := ResultValuesParsed + 'X,-,';
            end;
            I := I+4;
            if (I mod (2*WidthDim) = 1) then begin
              ResultValuesParsed[Length(ResultValuesParsed)] := '|';
              I := I + (2*WidthDim);
            end;
          end;
        end;

        for I := 1 to Length(ResultValuesParsed) do begin
          if (I mod 4 = 0) and not (I mod WidthDim = 0) then begin
            ResultValuesParsed[I] := ';';
          end;
        end;
        SetLength(ResultValuesParsed, (WidthDim*HeightDim));
        Delete(ResultValuesParsed,Length(ResultValuesParsed),1);
        // Kraj parsiranja

        //LogMessage('RR - rezultati: '+ResultValuesParsed);
        Result := ResultValuesParsed;
end;

end.
